package edu.sc.seis.darkMagic;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Properties;

import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.LocalObject;
import org.omg.IOP.ServiceContext;
import org.omg.PortableInterceptor.Current;
import org.omg.PortableInterceptor.ForwardRequest;
import org.omg.PortableInterceptor.ServerRequestInfo;
import org.omg.PortableInterceptor.ServerRequestInterceptor;

import edu.iris.Fissures.IfNetwork.NetworkId;
import edu.iris.Fissures.IfSeismogramDC.DataCenterHelper;
import edu.iris.Fissures.IfSeismogramDC.RequestFilter;
import edu.iris.Fissures.model.MicroSecondDate;
import edu.iris.Fissures.model.QuantityImpl;
import edu.iris.Fissures.model.UnitImpl;
import edu.iris.Fissures.network.ChannelIdUtil;
import edu.iris.Fissures.network.NetworkIdUtil;
import edu.sc.seis.fissuresUtil.Alohomora;
import edu.sc.seis.fissuresUtil.display.MicroSecondTimeRange;

/**
 * @author crotwell Created on Feb 10, 2005
 */
public class Fluffy extends LocalObject implements ServerRequestInterceptor {

    public Fluffy(Current piCurrent, int outCallIndicatorSlotId) {
        // we make the assumption that there will be only one instance of this
        // class
        // at a time, probably ok as interceptors are per orb.
        singleton = this;
        logger.warn("Fluffy on duty");
    }

    /**
     * This is a dumb implementation that says all X?, Y? and Z? networks that
     * began within 4 years of now are restricted. This should likely come from
     * some internal DMC data source.
     */
    public boolean isRestricted(NetworkId net) {
        logger.warn("Check restricted: " + NetworkIdUtil.toStringFormatDates(net));
        if (NetworkIdUtil.isTemporary(net)) {
            MicroSecondDate netBegin = new MicroSecondDate(net.begin_time);
            MicroSecondDate now = new MicroSecondDate();
            return now.subtract(netBegin).lessThan(FOUR_YEARS);
        }
        return false;
    }

    public void receive_request_service_contexts(ServerRequestInfo info) throws ForwardRequest {
        logger.debug("fluffy receive_request_service_contexts: " + info.operation());
    }

    public void receive_request(ServerRequestInfo info) throws ForwardRequest {
        try {
            logger.debug("fluffy receive_request:" + info.operation());
            if (info.target_is_a(DataCenterHelper.id())) {
                logger.debug("is a datacenter");
                Properties passwordProps = new Properties();
                try {
                    ServiceContext context = info.get_request_service_context(Alohomora.ID);
                    ByteArrayInputStream in = new ByteArrayInputStream(context.context_data);
                    passwordProps.load(in);
                } catch(BAD_PARAM e) {
                    // likely means that no passwords were inserted on the
                    // client side, so leave passwords props as empty
                    logger.debug("No passwords from client");
                }
                threadLocalPasswords = new ThreadLocalPassword(passwordProps);
            }
        } catch(IOException e) {
            logger.error("this shouldn't happen as we are loading from a byte array", e);
            throw new RuntimeException(e);
        } catch(RuntimeException e) {
            logger.error("problem in Fluffy:", e);
            throw e;
        }
    }

    public void send_reply(ServerRequestInfo arg0) {}

    public void send_exception(ServerRequestInfo arg0) throws ForwardRequest {}

    public void send_other(ServerRequestInfo arg0) throws ForwardRequest {}

    public String name() {
        return "Fluffy: Cerberus Corbarus";
    }

    public void destroy() {}

    public RequestFilter[] checkPermission(RequestFilter[] inFilters) {
        RequestFilter[] filters = controller.expandNetWildCards(inFilters);
        ArrayList out = new ArrayList();
        for (int i = 0; i < filters.length; i++) {
            logger.debug("check: " + printRF(filters[i]));
            NetworkId net = filters[i].channel_id.network_id;
            if (!isRestricted(net)) {
                logger.debug("ok, not restricted" + printRF(filters[i]));
                out.add(filters[i]);
            } else {
                String key = darkMagic + "." + formatNetworkYear(net);
                if (!passwords.containsKey(key)) {
                    // restricted, but no password in props, reject
                    logger.debug("reject " + printRF(filters[i]) + ", no password in props for " + key);
                } else {
                    String expectedPass = passwords.getProperty(key);
                    Properties clientPasswords = (Properties)threadLocalPasswords.get();
                    if (clientPasswords != null) {
                        if (clientPasswords.containsKey(key)) {
                            if (clientPasswords.getProperty(key).equals(expectedPass)) {
                                logger.debug("ok, passwords match " + printRF(filters[i]));
                                out.add(filters[i]);
                            } else {
                                logger.debug("reject " + printRF(filters[i]) + " Wrong password from client");
                            }
                            // OK to go on
                        } else {
                            logger.debug("reject " + printRF(filters[i]) + " No password for net from client");
                        }
                    } else {
                        logger.debug("reject" + printRF(filters[i]) + " No passwords from client");
                    }
                }
            }
        }
        return (RequestFilter[])out.toArray(new RequestFilter[0]);
    }

    public static String printRF(RequestFilter rf) {
        return ChannelIdUtil.toStringFormatDates(rf.channel_id) + " " + new MicroSecondTimeRange(rf);
    }

    public static String formatNetworkYear(NetworkId netId) {
        return netId.network_code + yearDateFormat.format(new MicroSecondDate(netId.begin_time));
    }

    public static void setController(FluffyController wrappedController) {
        controller = wrappedController;
    }

    public static Fluffy getFluffy() {
        return singleton;
    }

    public static void setPasswordProps(Properties passwordProps) {
        passwords = passwordProps;
    }

    private ThreadLocal threadLocalPasswords;

    private static Properties passwords = new Properties();

    private static Fluffy singleton;

    private static FluffyController controller;

    public static final QuantityImpl FOUR_YEARS = new QuantityImpl(4, UnitImpl.GREGORIAN_YEAR);

    public static final String darkMagic = "darkMagic";

    public static SimpleDateFormat yearDateFormat = new SimpleDateFormat("yyyy");

    private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(Fluffy.class);
}

class ThreadLocalPassword extends ThreadLocal {

    public ThreadLocalPassword(Properties passwordProps) {
        this.passwordProps = passwordProps;
    }

    Properties passwordProps;

    protected synchronized Object initialValue() {
        return passwordProps;
    }
}
